import uuid
import json

from django.conf import settings
from rest_framework.test import APITestCase, APIClient

from djangoldp_account.models import LDPUser
from djangoldp_notification.models import Notification, Subscription


class SubscriptionTestCase(APITestCase):
    def setUp(self):
        self.client = APIClient()

    def _get_random_user(self):
        return LDPUser.objects.create(email='{}@test.co.uk'.format(str(uuid.uuid4())), first_name='Test',
                                      last_name='Test', username=str(uuid.uuid4()))

    def _auth_as_user(self, user):
        self.client.force_authenticate(user=user)

    def setUpLoggedInUser(self):
        self.user = self._get_random_user()
        self._auth_as_user(self.user)

    def test_can_subscribe_to_container(self):
        self.setUpLoggedInUser()

        users_container = "{}/users/".format(settings.SITE_URL)
        Subscription.objects.create(object=users_container, inbox=self.user.urlid + "inbox/")
        self.assertEqual(self.user.inbox.count(), 0)

        self._get_random_user()
        self.assertEqual(self.user.inbox.count(), 2)
        notification = self.user.inbox.all()[0]
        self.assertTrue(notification.object, users_container)



# Generated by Django 2.2.23 on 2021-10-16 10:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_notification', '0013_auto_20211016_1203'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='disable_automatic_notifications',
            field=models.BooleanField(default=False, help_text='By default, notifications will be sent to this inbox everytime the target object/container is updated. Setting this flag to true prevents this behaviour, meaning that notifications will have to be triggered manually'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-08-28 12:01
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('author', djangoldp.fields.LDPUrlField()),
                ('object', djangoldp.fields.LDPUrlField()),
                ('type', models.CharField(max_length=255)),
                ('summary', models.TextField()),
                ('date', models.DateTimeField(auto_now_add=True)),
                ('unread', models.BooleanField(default=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='inbox', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-date'],
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'control'),
            },
        ),
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object', models.URLField()),
                ('inbox', models.URLField()),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'control'),
            },
        ),
    ]



import json
import argparse
from pathlib import Path
from copy import deepcopy
from datetime import datetime

'''
A script which generates and outputs placeholder notifications attached to a parametrised user, output 
into a parameterised file (json), which can be used as a Django fixture or imported into a live database
e.g. python manage.py loaddata fixture.json
for help run python generate_inbox_fixture.py -h
'''

# starting from offset ensures that existing users etc are not disturbed
parser = argparse.ArgumentParser(description='generates and outputs random test data, into a file used by the performance unit tests')
parser.add_argument(dest='count', metavar='N', type=int, help='the number of users (and projects) to generate')
parser.add_argument(dest='user', metavar='U', type=str, help='the primary key of the user whose inbox should store the notifications')
parser.add_argument('--offset', dest='offset', type=int, default=100, help='an offset to start primary keys at (should be larger than the largest pre-existing project/user primary key)')
parser.add_argument('-f', dest='file_dest', type=str, default="../fixtures/inbox.json", help='the file destination to write to')

args = parser.parse_args()
count = args.count
user = args.user
OFFSET = args.offset

notification_template = {
    'model': 'djangoldp_notification.notification',
    'pk': 0,
    'fields': {
        'user': user,
        'author': 'Test',
        'object': 'http://localhost:8000/users/admin/',
        'type': 'Update',
        'summary': 'Test',
        'date': str(datetime.date(datetime.now()))
    }
}

fixture = list()
for i in range(count):
    notification = deepcopy(notification_template)
    notification['pk'] = OFFSET + i
    fixture.append(notification)

with open(Path(__file__).parent / args.file_dest, 'w') as output:
    json.dump(fixture, output)

print(str(count))



from django.contrib import admin
from djangoldp.admin import DjangoLDPAdmin
from djangoldp.models import Model
from .models import Notification, NotificationSetting, Subscription


class NotificationAdmin(DjangoLDPAdmin):
    list_display = ('urlid', 'user', 'author', 'type', 'object', 'unread')
    exclude = ('urlid', 'is_backlink', 'allow_create_backlink')
    search_fields = ['urlid', 'user__urlid', 'author', 'object', 'type', 'summary']
    ordering = ['-urlid']

    def get_queryset(self, request):
        # Hide distant notification
        queryset = super(NotificationAdmin, self).get_queryset(request)
        internal_ids = [x.pk for x in queryset if not Model.is_external(x)]
        return queryset.filter(pk__in=internal_ids)


class SubscriptionAdmin(DjangoLDPAdmin):
    list_display = ('urlid', 'object', 'inbox', 'field')
    exclude = ('urlid', 'is_backlink', 'allow_create_backlink')
    search_fields = ['urlid', 'object', 'inbox', 'field']
    ordering = ['urlid']


class NotificationSettingAdmin(DjangoLDPAdmin):
    list_display = ('urlid', 'user', 'receiveMail')
    exclude = ('urlid', 'is_backlink', 'allow_create_backlink')
    search_fields = ['urlid', 'user', 'receiveMail']
    ordering = ['urlid']


admin.site.register(Notification, NotificationAdmin)
admin.site.register(Subscription, SubscriptionAdmin)
admin.site.register(NotificationSetting, NotificationSettingAdmin)



import uuid
import json
from rest_framework.test import APITestCase, APIClient

from django.conf import settings
from djangoldp.models import Model
from djangoldp.serializers import GLOBAL_SERIALIZER_CACHE
from djangoldp_account.models import LDPUser
from djangoldp_notification.models import Notification


class TestSubscription(APITestCase):
    def _get_random_user(self):
        return LDPUser.objects.create(email='{}@test.co.uk'.format(str(uuid.uuid4())), first_name='Test',
                                      last_name='Test', username=str(uuid.uuid4()))

    def _get_random_notification(self, recipient, author):
        return Notification.objects.create(user=recipient, author=author.urlid, object=author.urlid,
                                           unread=True)

    def setUpLoggedInUser(self):
        self.user = self._get_random_user()
        self.client.force_authenticate(user=self.user)

    def setUp(self):
        self.client = APIClient()

    def test_indirect_cache(self):
        self.setUpLoggedInUser()
        author_user = self._get_random_user()
        notification = self._get_random_notification(recipient=self.user, author=author_user)
        self.assertEqual(notification.unread, True)

        # GET the inbox - should set the cache
        response = self.client.get("/users/{}/inbox/".format(self.user.username))
        self.assertEqual(response.status_code, 200)
        notif_serialized = response.data["ldp:contains"][0]
        self.assertEqual(notif_serialized["unread"], True)

        # PATCH the notification - should wipe the cache
        patch = {
            "unread": False,
            "@context": {
                "@vocab":"http://happy-dev.fr/owl/#",
                "unread": "http://happy-dev.fr/owl/#unread"
            }
        }
        response = self.client.patch("/notifications/{}/".format(notification.pk), data=json.dumps(patch),
                                     content_type="application/ld+json")
        notif_obj = Notification.objects.get(pk=notification.pk)
        self.assertEqual(notif_obj.unread, False)

        # GET the inbox - should now be read
        response = self.client.get("/users/{}/inbox/".format(self.user.username))
        self.assertEqual(response.status_code, 200)
        notif_serialized = response.data["ldp:contains"][0]
        self.assertEqual(notif_serialized["unread"], False)

    # NOTE: this would be our ideal cache behaviour
    # the functionality for optimising it was removed because of an issue with extensibility
    #  https://git.startinblox.com/djangoldp-packages/djangoldp-notification/merge_requests/42#note_58559
    '''def test_custom_cache_clear(self):
        # going to create two notifications in two different inboxes
        self.setUpLoggedInUser()
        other_user = self._get_random_user()
        notification = self._get_random_notification(recipient=self.user, author=other_user)
        notification2 = self._get_random_notification(recipient=other_user, author=self.user)

        # GET the inboxes and asser that the cache is set for both
        self.client.get("/users/{}/inbox/".format(self.user.username))
        self.client.get("/users/{}/inbox/".format(other_user.username))

        # assert cache is set
        my_container_urlid = '{}/users/{}/inbox/'.format(settings.SITE_URL, self.user.username)
        their_container_urlid = '{}/users/{}/inbox/'.format(settings.SITE_URL, other_user.username)

        self.assertTrue(GLOBAL_SERIALIZER_CACHE.has(Model.get_meta(Notification, 'label'), my_container_urlid))
        self.assertTrue(GLOBAL_SERIALIZER_CACHE.has(Model.get_meta(Notification, 'label'), their_container_urlid))

        # save my notification - should wipe the cache for my inbox...
        notification.unread = False
        notification.save()
        self.assertFalse(GLOBAL_SERIALIZER_CACHE.has(Model.get_meta(Notification, 'label'), my_container_urlid))

        # ...but not for theirs
        self.assertTrue(GLOBAL_SERIALIZER_CACHE.has(Model.get_meta(Notification, 'label'), their_container_urlid))'''



# Generated by Django 2.2.23 on 2021-07-29 17:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_notification', '0011_auto_20210218_1145'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='summary',
            field=models.TextField(blank=True, default=''),
        ),
    ]



from django.core.management.base import BaseCommand
from django.conf import settings
from djangoldp_notification.models import Subscription

class Command(BaseCommand):
  help = 'Create required subscriptions'

  def handle(self, *args, **options):
    host = getattr(settings, 'SITE_URL')
    jabber_host = getattr(settings, 'JABBER_DEFAULT_HOST')
    xmpp = getattr(settings, 'PROSODY_HTTP_URL')

    try:
        sub = Subscription.objects.get(object="{}/circles/".format(host))
        sub.delete()
    except Subscription.DoesNotExist:
        pass

    try:
        sub = Subscription.objects.get(object="{}/projects/".format(host))
        sub.delete()
    except Subscription.DoesNotExist:
        pass

    try:
        sub = Subscription.objects.get(object="{}/circle-members/".format(host))
        sub.delete()
    except Subscription.DoesNotExist:
        pass

    try:
        sub = Subscription.objects.get(object="{}/project-members/".format(host))
        sub.delete()
    except Subscription.DoesNotExist:
        pass

    Subscription.objects.get_or_create(object=host+"/circles/", inbox=xmpp + "/conference." + jabber_host + "/startinblox_muc_admin", field=None)
    Subscription.objects.get_or_create(object=host+"/projects/", inbox=xmpp + "/conference." + jabber_host + "/startinblox_muc_admin", field=None)
    
    Subscription.objects.get_or_create(object=host+"/users/", inbox=xmpp + "/" + jabber_host + "/happydev_user_admin", field=None)
    Subscription.objects.get_or_create(object=host+"/profiles/", inbox=xmpp + "/" + jabber_host + "/happydev_user_admin", field="user")
    Subscription.objects.get_or_create(object=host+"/chatprofiles/", inbox=xmpp + "/" + jabber_host + "/happydev_user_admin", field="user")
    Subscription.objects.get_or_create(object=host+"/accounts/", inbox=xmpp + "/" + jabber_host + "/happydev_user_admin", field="user")

    Subscription.objects.get_or_create(object=host+"/circle-members/", inbox=xmpp + "/conference." + jabber_host + "/startinblox_muc_members_admin", field=None)
    Subscription.objects.get_or_create(object=host+"/project-members/", inbox=xmpp + "/conference." + jabber_host + "/startinblox_muc_members_admin", field=None)

    self.stdout.write(self.style.SUCCESS("Successfully  created subscriptions\nhost: "+host+"\nxmpp server: "+xmpp+"\njabber host: "+jabber_host))



# Generated by Django 2.2.16 on 2020-11-16 09:19

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('djangoldp_notification', '0009_auto_20200619_0802'),
    ]

    def create_settings(apps, schema_editor):
        Users = apps.get_model('djangoldp_account', 'LDPUser')
        Settings = apps.get_model('djangoldp_notification', 'NotificationSetting')

        # iterate over all objects
        for user in Users.objects.all():
            if user.urlid.startswith(settings.SITE_URL):
                Settings.objects.create(user=user)

    operations = [
        migrations.CreateModel(
            name='NotificationSetting',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('receiveMail', models.BooleanField(default=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='settings', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.RunPython(create_settings),
    ]



import sys
import yaml

import django
from django.conf import settings as django_settings
from djangoldp.conf.ldpsettings import LDPSettings
from djangoldp.tests.server_settings import yaml_config

# override config loading
config = {
    # add the packages to the reference list
    'ldppackages': ['djangoldp_account', 'djangoldp_notification', 'djangoldp_notification.tests'],

    # required values for server
    'server': {
        'AUTH_USER_MODEL': 'djangoldp_account.LDPUser',
        'REST_FRAMEWORK': {
            'DEFAULT_PAGINATION_CLASS': 'djangoldp.pagination.LDPPagination',
            'PAGE_SIZE': 5
        },
        # map the config of the core settings (avoid asserts to fail)
        'SITE_URL': 'http://happy-dev.fr',
        'BASE_URL': 'http://happy-dev.fr',
        'SEND_BACKLINKS': False,
        'JABBER_DEFAULT_HOST': None,
        'PERMISSIONS_CACHE': False,
        'ANONYMOUS_USER_NAME': None,
        'SERIALIZER_CACHE': True,
        'USER_NESTED_FIELDS': ['inbox', 'settings'],
        'USER_EMPTY_CONTAINERS': ['inbox'],
        'EMAIL_BACKEND': 'django.core.mail.backends.console.EmailBackend'
    }
}
ldpsettings = LDPSettings(config)
ldpsettings.config = yaml.safe_load(yaml_config)

django_settings.configure(ldpsettings)

django.setup()
from django.test.runner import DiscoverRunner

test_runner = DiscoverRunner(verbosity=1)

failures = test_runner.run_tests([
    'djangoldp_notification.tests.test_models',
    'djangoldp_notification.tests.test_subscriptions',
    'djangoldp_notification.tests.test_cache',
])
if failures:
    sys.exit(failures)



from django.conf import settings
from django.contrib.auth import get_user_model
from django.http import Http404
from rest_framework import status
from rest_framework.response import Response

from djangoldp.models import Model
from djangoldp.serializers import LDPSerializer
from djangoldp.views import LDPViewSet
from djangoldp.pagination import LDPPagination
import logging


logger = logging.getLogger('djangoldp')


class LDPNotificationsPagination(LDPPagination):
    default_limit = 80


def filter_object_is_permitted(recipient, data):
        '''
        applies filter on passed object data
        returns True if the object is permitted, False if not
        '''
        obj = data['object']

        # if a str (urlid) is given then no filtering is to be applied
        if isinstance(obj, str):
            return True

        # the type must be given for a filter to be resolved successfully
        if not '@type' in obj or obj['@type'] is None:
            logger.error('djangoldp_notification filter ERR in object serialization. received ' + str(obj) + ' without serialized type to identify model. Please include the @type in your serialization')
            return False

        object_model = Model.get_subclass_with_rdf_type(obj['@type'])

        if object_model is None:
            logger.error('djangoldp_notification filter ERR in object serialization. Cannot resolve type given ' + str(obj['type']))
            return False
        
        if not hasattr(object_model, 'permit_notification'):
            logger.error('djangoldp_notification filter ERR. Resolved type ' + str(obj['@type']) + ' but this Model did not have the required function permit_notification defined on it')
            return False

        return object_model.permit_notification(recipient, data)


class LDPNotificationsViewSet(LDPViewSet):
    '''overridden LDPViewSet to force pagination'''
    pagination_class = LDPNotificationsPagination
    depth = 0



# Generated by Django 2.2.23 on 2021-10-16 10:03

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_notification', '0012_auto_20210729_1912'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='author',
            field=djangoldp.fields.LDPUrlField(help_text='the sender of the notification'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='object',
            field=djangoldp.fields.LDPUrlField(help_text='the urlid of the saved object being transmitted by the notification'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='unread',
            field=models.BooleanField(default=True, help_text='set to False after the user has seen the notification'),
        ),
        migrations.AlterField(
            model_name='notification',
            name='user',
            field=models.ForeignKey(help_text='the recipient of the notification', on_delete=django.db.models.deletion.CASCADE, related_name='inbox', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='notificationsetting',
            name='receiveMail',
            field=models.BooleanField(default=True, help_text='if set to True the user will receive an email on notification receipt'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='inbox',
            field=models.URLField(help_text='the inbox of the recipient of the notification'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='object',
            field=models.URLField(help_text='the urlid of the object being subscribed'),
        ),
    ]
